//
// This file is part of the GNU ARM Eclipse distribution.
// Copyright (c) 2014 Liviu Ionescu.
//

// ----------------------------------------------------------------------------

#include <stdio.h>
#include "diag/Trace.h"
#include "stm32f10x.h"
#include "LiquidCrystal_I2C.h"
#include <vector>

// ----------------------------------------------------------------------------
//
// STM32F1 empty sample (trace via ITM).
//
// Trace support is enabled by adding the TRACE macro definition.
// By default the trace messages are forwarded to the ITM output,
// but can be rerouted to any device or completely suppressed, by
// changing the definitions required in system/src/diag/trace_impl.c
// (currently OS_USE_TRACE_ITM, OS_USE_TRACE_SEMIHOSTING_DEBUG/_STDOUT).
//

// ----- main() ---------------------------------------------------------------

// Sample pragmas to cope with warnings. Please note the related line at
// the end of this function, used to pop the compiler diagnostics status.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#pragma GCC diagnostic ignored "-Wmissing-declarations"
#pragma GCC diagnostic ignored "-Wreturn-type"

GPIO_InitTypeDef GPIO_InitStructure;
USART_InitTypeDef USART_InitStruct;
TIM_TimeBaseInitTypeDef TIM_TimeBase_InitStruct;
//TIM_OCInitTypeDef TIM_OC_InitStruct;
NVIC_InitTypeDef NVIC_InitStruct;
I2C_InitTypeDef I2C_InitStruct;

uint8_t rx_arr[8] = { 0, };
uint8_t tx_arr[8] = { 0, };
uint16_t recv = 0;
volatile uint16_t tick_ms = 0;
volatile uint16_t tick_us = 0;

/**
 * delay of delay milliseconds
 * @param delay
 */
void delay_ms(uint16_t delay) {
    uint16_t tick_start = tick_ms;
    if (delay > 50000) {
        delay = 50000;
    }
    while ((tick_ms - tick_start) < delay)
        ;
}

/**
 * delay of delay*10 microseconds
 * @param delay
 */
void delay_10us(uint16_t delay) {
    uint16_t tick_start = tick_us;
    if (delay > 50000) {
        delay = 50000;
    }
    while ((tick_us - tick_start) < delay)
        ;
}

void toggleBit(GPIO_TypeDef* port, uint16_t pin) {
    port->BSRR = ((port->ODR ^ pin) & pin) | (pin << 16);
}

uint8_t usart_sendString(USART_TypeDef* USARTx, const char* str) {
    uint8_t cnt = 0;
    while (*str) {
        if (USART_GetFlagStatus(USARTx, USART_FLAG_TXE) == SET) {
            USART_SendData(USARTx, *str);
            str++;
            cnt++;
        }
    }
    return cnt;
}

int main(int argc, char* argv[]) {
    SystemInit();

    RCC_APB2PeriphClockCmd(
    RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
    RCC_APB1PeriphClockCmd(
    RCC_APB1Periph_USART2 | RCC_APB1Periph_TIM2 | RCC_APB1Periph_I2C2, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_0
            | GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
//    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);


    USART_InitStruct.USART_BaudRate = 9600;
    USART_InitStruct.USART_WordLength = USART_WordLength_8b;
    USART_InitStruct.USART_StopBits = USART_StopBits_1;
    USART_InitStruct.USART_Parity = USART_Parity_No;
    USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStruct.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
    USART_Init(USART2, &USART_InitStruct);

    USART_Cmd(USART2, ENABLE);

//    USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);

    TIM_TimeBase_InitStruct.TIM_ClockDivision = TIM_CKD_DIV1;
    TIM_TimeBase_InitStruct.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBase_InitStruct.TIM_Period = 710;
    TIM_TimeBase_InitStruct.TIM_Prescaler = 0;
    TIM_TimeBaseInit(TIM2, &TIM_TimeBase_InitStruct);

    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

//    TIM_OC_InitStruct.TIM_OCMode = TIM_OCMode_PWM1;
//    TIM_OC_InitStruct.TIM_OCIdleState = TIM_OCIdleState_Reset;
//    TIM_OC_InitStruct.TIM_OCNIdleState = TIM_OCNIdleState_Set;
//    TIM_OC_InitStruct.TIM_OCPolarity = TIM_OCPolarity_High;
//    TIM_OC_InitStruct.TIM_OCNPolarity = TIM_OCNPolarity_High;
//    TIM_OC_InitStruct.TIM_OutputState = TIM_OutputState_Enable;
//    TIM_OC_InitStruct.TIM_OutputNState = TIM_OutputNState_Disable;
//    TIM_OC_InitStruct.TIM_Pulse = 500;
//    TIM_OC2Init(TIM2, &TIM_OC_InitStruct);

    NVIC_InitStruct.NVIC_IRQChannel = TIM2_IRQn;
    NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
    NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 0x0F;
    NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0x0F;
    NVIC_Init(&NVIC_InitStruct);

    TIM_Cmd(TIM2, ENABLE);

// ###################### I2C
//    NVIC_InitStruct.NVIC_IRQChannel = I2C2_EV_IRQn;
//    NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 0;
//    NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0;
//    NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
//    NVIC_Init(&NVIC_InitStruct);
//
//    NVIC_InitStruct.NVIC_IRQChannel = I2C2_ER_IRQn;
//    NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 0;
//    NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0;
//    NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
//    NVIC_Init(&NVIC_InitStruct);

    I2C_DeInit(I2C2);

    I2C_InitStruct.I2C_Ack = I2C_Ack_Enable;
    I2C_InitStruct.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    I2C_InitStruct.I2C_ClockSpeed = 100000;
    I2C_InitStruct.I2C_DutyCycle = I2C_DutyCycle_2;
    I2C_InitStruct.I2C_Mode = I2C_Mode_I2C;
    I2C_InitStruct.I2C_OwnAddress1 = 0;
    I2C_Init(I2C2, &I2C_InitStruct);

//    I2C_ITConfig(I2C2, I2C_IT_EVT, ENABLE);
//    I2C_ITConfig(I2C2, I2C_IT_BUF, ENABLE);
//    I2C_ITConfig(I2C2, I2C_IT_ERR, ENABLE);

    I2C_Cmd(I2C2, ENABLE);

    uint16_t old_tick = 0;
    uint16_t x = 0;

    LiquidCrystal_I2C lcd = LiquidCrystal_I2C(I2C2, 0x27, 20, 4);
    lcd.init();
    lcd.backlight();
    lcd.cursor();
    lcd.blink();
        lcd.setCursor(0,0);
        lcd.printstr("Hello, world!");
//    i2c_create(I2C2);
//    for (int i = 0x01; i < 0x30; i++) {
////        i2c_writeByte(i, 0xFF);
//        i2c_writeTwoBytes(i, 0xFF, 0xFF);
//        i2c_readTwoBytes(i);
//        x = i2c_getData();
//        while (USART_GetFlagStatus(USART2, USART_FLAG_TXE) != SET)
//            ;
//        USART_SendData(USART2, i);
//        while (USART_GetFlagStatus(USART2, USART_FLAG_TXE) != SET)
//            ;
//        USART_SendData(USART2, x >> 8);
//        while (USART_GetFlagStatus(USART2, USART_FLAG_TXE) != SET)
//            ;
//        USART_SendData(USART2, x);
//        while (USART_GetFlagStatus(USART2, USART_FLAG_TXE) != SET)
//            ;
//        USART_SendData(USART2, '\n');
//        delay_ms(10);
//    }
//    lcd.backlight();
//    lcd.setCursor(3,0);
//    lcd.printstr("Hello, world!");
//    lcd.setCursor(3,1);
//    lcd.printstr("YwRobot Studio");
//    lcd.setCursor(0,2);
//    lcd.printstr("<-LCM IIC 2004 V2->");
//    lcd.setCursor(2,3);
//    lcd.printstr("*==> (^_^) <==*");
// Infinite loop
    for (;;) {
//        GPIOA->BSRR = GPIO_Pin_5;
//        GPIOA->BRR = GPIO_Pin_1;
//        GPIOA->BRR = GPIO_Pin_5;
//        GPIOA->BSRR = GPIO_Pin_1;
//        if (USART_GetFlagStatus(USART2, USART_FLAG_TXE) == RESET) {
//            GPIOA->BRR = GPIO_Pin_5;
//        usart_sendString(USART2, "hello world!\n");

//        while (USART_GetFlagStatus(USART2, USART_FLAG_RXNE) == RESET) {
//        }
//        recv = USART_ReceiveData(USART2);
//        USART_SendData(USART2, 'x');

//        if (USART_GetFlagStatus(USART2, USART_FLAG_RXNE) == SET) {
//            if (GPIO_ReadOutputDataBit(GPIOA, GPIO_Pin_5)) {
//                GPIO_ResetBits(GPIOA, GPIO_Pin_5);
//            } else {
//                GPIO_SetBits(GPIOA, GPIO_Pin_5);
//            }
//            USART_ClearFlag(USART2, USART_FLAG_RXNE);
//        }
//        USART_SendBreak(USART2);
//        } else {
//            GPIOA->BSRR = GPIO_Pin_5;
//        }
//       for (uint16_t i = 0; i < 1000; i += 1) {
//            TIM_SetCounter(TIM2, 0xFFEE);
//           TIM_SetAutoreload(TIM2, 0xFFEE);
//           delayLoop();
//        }
        if (tick_ms < old_tick) {
            old_tick = 0;
        }
        if ((tick_ms - old_tick) > 250) {
            old_tick = tick_ms;
//            USART_SendData(USART2, 'x');
            toggleBit(GPIOA, GPIO_Pin_5);
//            if (GPIOA->BSRR & GPIO_Pin_5) {
//                GPIO_ResetBits(GPIOA, GPIO_Pin_5);
//            } else {
//                GPIO_SetBits(GPIOA, GPIO_Pin_5);
//            }
        }

    }
}

extern "C" void TIM2_IRQHandler(void) {
    TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
    if (tick_us > 60000) {
        tick_us = 0;
    } else {
        tick_us++;
    }
    if ((tick_us % 100) == 0) {
        if (tick_ms > 60000) {
            tick_ms = 0;
        } else {
            tick_ms++;
        }
    }
}

//extern "C" void I2C2_EV_IRQHandler(void) {
//    i2c_handleEventInterrupt();
////            USART_SendData(USART2, 'a');
//}
//
//extern "C" void I2C2_ER_IRQHandler(void) {
//    i2c_handleErrorInterrupt();
////            USART_SendData(USART2, 'b');
//}

#pragma GCC diagnostic pop

// ----------------------------------------------------------------------------
